import { Transformer, Supplier, BiConsumer, BiFunction } from "../functions";
import { MutableString, MutableNumber } from './mutableCollections';
/**
 * A mutable reduction operation that accumulates input elements into a mutable result container,
 * optionally transforming the accumulated result into a final representation after all input elements have been processed.
 *
 * Examples of mutable reduction operations include: accumulating elements into an array; concatenating strings;
 * computing summary information about elements such as sum, min, max, or average;
 * computing "pivot table" summaries such as "maximum valued transaction by seller", etc.
 *
 * Collectors provides implementations of many common mutable reductions.
 *
 * A Collector is specified by four functions that work together to accumulate entries into a mutable result container,
 * and optionally perform a final transform on the result. They are:
 * creation of a new result container (supplier())
 * incorporating a new data element into a result container (accumulator())
 * combining two result containers into one (combiner())
 * performing an optional final transform on the container (finisher())
 */
export declare class Collector<T, A, R> {
    private supp;
    private accu;
    private comb;
    private fini;
    private constructor();
    supplier: () => Supplier<A>;
    accumulator: () => BiConsumer<A, T>;
    combiner: () => BiFunction<A>;
    finisher: () => Transformer<A, R>;
    /**
    * A Collector is specified by four functions that work together to accumulate entries into a mutable result container,
    * and optionally perform a final transform on the result. They are:
    * @param supplier function that returns of a new result container
    * @param accumulator function for incorporating a new data element into a result container
    * @param combiner function for combining two result containers into one
    * @param finisher function for performing an optional final transform on the container
    */
    static of<T, A, R>(supplier: Supplier<A>, accumulator: BiConsumer<A, T>, combiner: BiFunction<A>, finisher: Transformer<A, R>): Collector<T, A, R>;
}
/**
 * defines many useful collectors for easily reducing/collecting data together
 */
declare class Collectors {
    /**
     * A collector that combines input into an array.
     */
    static toArray<T>(): Collector<T, T[], T[]>;
    /**
     * alias of toArray()
     */
    static toList<T>(): Collector<T, T[], T[]>;
    /**
     * Returns a collector that combines strings into a single long string, a delimiter string can be specified
     * to seperate individual items, the prefix and suffix are added to the resulting final string, each item.
     * @param delimiter: optional. Seperator for items being joined
     * @param prefix: optional. string added to the beginning of the joined result
     * @param suffix: optional. string added to the end of the joined result
     */
    static joining(delimiter?: string, prefix?: string, suffix?: string): Collector<string, MutableString, string>;
    /**
     * Returns a Collector that produces the arithmetic mean of a number-valued function applied to the input elements.
     * @param mapper Transformer function to transform input elements into a number
     */
    static averagingNumber<I>(mapper: Transformer<I, number>): Collector<I, MutableNumber, number>;
    /**
     * returns a Collector that produces the arithmetic mean of input numbers
     */
    static averaging(): Collector<number, MutableNumber, number>;
    static collectingAndThen<Input, Mutable, Intermediate, Output>(downStream: Collector<Input, Mutable, Intermediate>, finisher: Transformer<Intermediate, Output>): Collector<Input, Mutable, Output>;
}
export default Collectors;
