"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var functions_1 = require("../functions");
var mutableCollections_1 = require("./mutableCollections");
/**
 * A mutable reduction operation that accumulates input elements into a mutable result container,
 * optionally transforming the accumulated result into a final representation after all input elements have been processed.
 *
 * Examples of mutable reduction operations include: accumulating elements into an array; concatenating strings;
 * computing summary information about elements such as sum, min, max, or average;
 * computing "pivot table" summaries such as "maximum valued transaction by seller", etc.
 *
 * Collectors provides implementations of many common mutable reductions.
 *
 * A Collector is specified by four functions that work together to accumulate entries into a mutable result container,
 * and optionally perform a final transform on the result. They are:
 * creation of a new result container (supplier())
 * incorporating a new data element into a result container (accumulator())
 * combining two result containers into one (combiner())
 * performing an optional final transform on the container (finisher())
 */
var Collector = /** @class */ (function () {
    function Collector(supp, accu, comb, fini) {
        var _this = this;
        this.supplier = function () { return _this.supp; };
        this.accumulator = function () { return _this.accu; };
        this.combiner = function () { return _this.comb; };
        this.finisher = function () { return _this.fini; };
        this.supp = supp;
        this.accu = accu;
        this.comb = comb;
        this.fini = fini;
    }
    /**
    * A Collector is specified by four functions that work together to accumulate entries into a mutable result container,
    * and optionally perform a final transform on the result. They are:
    * @param supplier function that returns of a new result container
    * @param accumulator function for incorporating a new data element into a result container
    * @param combiner function for combining two result containers into one
    * @param finisher function for performing an optional final transform on the container
    */
    Collector.of = function (supplier, accumulator, combiner, finisher) {
        return new Collector(supplier, accumulator, combiner, finisher);
    };
    return Collector;
}());
exports.Collector = Collector;
;
/**
 * defines many useful collectors for easily reducing/collecting data together
 */
var Collectors = /** @class */ (function () {
    function Collectors() {
    }
    /**
     * A collector that combines input into an array.
     */
    Collectors.toArray = function () {
        var supplier = function () { return []; };
        var accumulator = function (list, item) { return list.push(item); };
        var combiner = function (list1, list2) { return list1.concat(list2); };
        var finisher = functions_1.Transformer.identity();
        return Collector.of(supplier, accumulator, combiner, finisher);
    };
    /**
     * alias of toArray()
     */
    Collectors.toList = function () {
        return Collectors.toArray();
    };
    /**
     * Returns a collector that combines strings into a single long string, a delimiter string can be specified
     * to seperate individual items, the prefix and suffix are added to the resulting final string, each item.
     * @param delimiter: optional. Seperator for items being joined
     * @param prefix: optional. string added to the beginning of the joined result
     * @param suffix: optional. string added to the end of the joined result
     */
    Collectors.joining = function (delimiter, prefix, suffix) {
        var ifElseBlank = function (val) { return val ? val : ""; };
        var delimiterToUse = ifElseBlank(delimiter);
        var supplier = function () { return mutableCollections_1.MutableString.empty(); };
        var accumulator = function (mutable, str) { return mutable.append(str + delimiterToUse); };
        var combiner = function (mutable1, mutable2) { return mutable1.concat(mutable2); };
        var finisher = function (mutable) {
            var valueWithExtraDelimiter = mutable.getValue();
            var valueWithOutExtraDelimiter = delimiterToUse.length === 0
                ? valueWithExtraDelimiter
                : valueWithExtraDelimiter.slice(0, -delimiterToUse.length);
            return ifElseBlank(prefix) + valueWithOutExtraDelimiter + ifElseBlank(suffix);
        };
        return Collector.of(supplier, accumulator, combiner, finisher);
    };
    /**
     * Returns a Collector that produces the arithmetic mean of a number-valued function applied to the input elements.
     * @param mapper Transformer function to transform input elements into a number
     */
    Collectors.averagingNumber = function (mapper) {
        var supplier = mutableCollections_1.MutableNumber.empty;
        var accumulator = function (mutable, item) { return mutable.add(mapper(item)); };
        var combiner = function (mNum1, mNum2) { return mNum1.addTogether(mNum2); };
        var finisher = function (mutable) {
            return mutable.getInputCount() > 0
                ? mutable.getTotal() / mutable.getInputCount()
                : 0;
        };
        return Collector.of(supplier, accumulator, combiner, finisher);
    };
    /**
     * returns a Collector that produces the arithmetic mean of input numbers
     */
    Collectors.averaging = function () {
        return Collectors.averagingNumber(functions_1.Transformer.identity());
    };
    Collectors.collectingAndThen = function (downStream, finisher) {
        var newFinisher = function (input) { return finisher(downStream.finisher()(input)); };
        return Collector.of(downStream.supplier(), downStream.accumulator(), downStream.combiner(), newFinisher);
    };
    return Collectors;
}());
;
exports.default = Collectors;
