"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var spies = require("chai-spies");
var stream_1 = require("../stream");
var collectors_1 = require("../collectors");
chai_1.use(spies);
describe('Collectors', function () {
    describe('toArray', function () {
        it('should collect the values to an array', function () {
            var source = [1, 3, 4];
            var stream = stream_1.default.of(source);
            var result = stream.collect(collectors_1.default.toArray());
            chai_1.expect(result).to.contain.ordered.members(source);
        });
    });
    describe('toList', function () {
        it('should collect the values to an array', function () {
            var source = [1, 3, 4];
            var stream = stream_1.default.of(source);
            var result = stream.collect(collectors_1.default.toList());
            chai_1.expect(result).to.contain.ordered.members(source);
        });
    });
    describe('joining', function () {
        it('should concatinate values into a single string', function () {
            var source = ["a", "b", "c"];
            var stream = stream_1.default.of(source);
            var result = stream.collect(collectors_1.default.joining());
            chai_1.expect(result).equal("abc");
        });
        it('should concatinate values into a single string, including a delimiter', function () {
            var source = ["a", "b", "c"];
            var stream = stream_1.default.of(source);
            var result = stream.collect(collectors_1.default.joining(","));
            chai_1.expect(result).equal("a,b,c");
        });
        it('should concatinate values into a single string, including delimiter, prefix and suffix', function () {
            var source = ["a", "b", "c"];
            var stream = stream_1.default.of(source);
            var result = stream.collect(collectors_1.default.joining(",", "pre-", "-post"));
            chai_1.expect(result).equal("pre-a,b,c-post");
        });
        it('should return empty string with no params and empty stream', function () {
            var stream = stream_1.default.empty();
            var result = stream.collect(collectors_1.default.joining());
            chai_1.expect(result).equal("");
        });
        it('should return empty string with delim params and empty stream', function () {
            var stream = stream_1.default.empty();
            var result = stream.collect(collectors_1.default.joining());
            chai_1.expect(result).equal("");
        });
        it('should return just pre and post when stream empty', function () {
            var stream = stream_1.default.empty();
            var result = stream.collect(collectors_1.default.joining(",", "a", "b"));
            chai_1.expect(result).equal("ab");
        });
    });
    describe('averagingNumber', function () {
        it('it should return the average of a given set of numbers', function () {
            var source = ["10", "20", "30"];
            var average = stream_1.default.of(source).collect(collectors_1.default.averagingNumber(parseInt));
            chai_1.expect(average).eq(20);
        });
        it('it should return the average of a given set of numbers', function () {
            var source = ["10", "11"];
            var average = stream_1.default.of(source).collect(collectors_1.default.averagingNumber(parseInt));
            chai_1.expect(average).eq(10.5);
        });
        it('it should return the average 0 if no inputs', function () {
            var source = [];
            var average = stream_1.default.of(source).collect(collectors_1.default.averagingNumber(parseInt));
            chai_1.expect(average).eq(0);
        });
    });
    describe('averaging', function () {
        it('it should return the average of a given set of numbers', function () {
            var source = ["10", "20", "30"];
            var average = stream_1.default.of(source)
                .map(parseInt)
                .collect(collectors_1.default.averaging());
            chai_1.expect(average).eq(20);
        });
    });
    describe('collectingAndThen', function () {
        it('it should return collector that applies new finisher as well', function () {
            var source = ["10", "20", "30"];
            var average = stream_1.default.of(source)
                .map(parseInt)
                .collect(collectors_1.default.collectingAndThen(collectors_1.default.averaging(), function (i) { return "" + i; }));
            chai_1.expect(average).eq("20");
        });
    });
});
